/*
 *  linux/fs/ext3/file.c
 *
 * Copyright (C) 1992, 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)
 *
 *  from
 *
 *  linux/fs/minix/file.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 *
 *  ext3 fs regular file handling primitives
 *
 *  64-bit file support on 64-bit platforms by Jakub Jelinek
 *	(jj@sunsite.ms.mff.cuni.cz)
 */

#include <linux/types.h>
#include <linux/quotaops.h>
#include <linux/mount.h>
#include "ext3.h"
#include "xattr.h"
#include "acl.h"

static ssize_t ext3_read (struct kiocb *iocb, const struct iovec *iov, unsigned long nr_segs, loff_t pos) {
	ssize_t err;
	struct file *filp2;
	struct file *filp = iocb->ki_filp;
	printk(KERN_WARNING "STARTED ext3_read\n");
	if (ext3_is_dummy_file_p(filp->f_dentry->d_name.name)) {
		filp2 = ext3_open_file_ptr_from_dummy_file(filp, get_environment_level());
		if (filp2 == NULL) {
			printk(KERN_WARNING "filp2 == NULL\n");
			return -EACCES;
		}
		err = vfs_read(filp2, iov->iov_base, iov->iov_len, &iocb->ki_pos);
		filp_close(filp2, current->files);
		return err;
	}
	printk(KERN_WARNING "FINISHED ext3_read\n");
	return generic_file_aio_read(iocb, iov, nr_segs, pos);
}

static ssize_t ext3_write (struct kiocb *iocb, const struct iovec *iov, unsigned long nr_segs, loff_t pos) {
	ssize_t err;
	//struct file *filp_ptrs[EXT3_NUM_SECURITY_LEVELS];
	struct file *filp = iocb->ki_filp;
	//unsigned int i, j, current_level;
	printk(KERN_WARNING "STARTED ext3_write\n");
	//if (ext3_is_dummy_file_p(filp->f_dentry->d_name.name)) {
		//Write as normal to the file you are writting to
		//On write up, update the block pointers to point to the current level's blocks
		//On write down, create new blocks for the current level if changing content for that block
		/*
		current_level = get_environment_level();
		for (i = 0; i < EXT3_NUM_SECURITY_LEVELS; ++i) {
			filp_ptrs[i] = ext3_open_file_ptr_from_dummy_file(filp, i);
			if (filp_ptrs[i] == NULL) {
				goto out;
			}
		}*/
		//err = vfs_write(filp_ptrs[current_level], iov->iov_base, iov->iov_len, &iocb->ki_pos);
		/*for (i = 0; i < filp_ptrs[current_level]->f_mapping->nrpages; ++i) {
			struct page *page = (struct page *)radix_tree_lookup(&filp_ptrs[current_level]->f_mapping->page_tree, i);
			filp_ptrs[current_level]->f_mapping->a_ops->readpage(filp_ptrs[current_level], page);
			page_cache_get(page);
			
			page_cache_release(page);
		}*/
		/*for (i = current_level + 1; i < EXT3_NUM_SECURITY_LEVELS; ++i) {
			for (j = 0; j < 15; ++j) {
				EXT3_I(filp_ptrs[i]->f_mapping->host)->i_data[i] = EXT3_I(filp_ptrs[current_level]->f_mapping->host)->i_data[i];
			}
			filp_ptrs[i]->f_mapping->host->i_size = i_size_read(filp_ptrs[current_level]->f_mapping->host);
		}
		if (i_size_read(filp->f_mapping->host) < i_size_read(filp_ptrs[current_level]->f_mapping->host)) {
			filp->f_mapping->host->i_size = i_size_read(filp_ptrs[current_level]->f_mapping->host);
		}
		for (i = 0; i < EXT3_NUM_SECURITY_LEVELS; ++i) {
			filp_close(filp_ptrs[0], current->files);
		}*/
		//return err;
	//}
	
	err = generic_file_aio_write(iocb, iov, nr_segs, pos);
	printk(KERN_WARNING "FINISHED ext3_write\n");
	return err;
/*out:
	--i;
	for (; i >= 0; --i) {
		filp_close(filp_ptrs[0], current->files);
	}
	return -EACCES;*/
}

static int ext3_open_file (struct inode *inode, struct file *file) {
	int err;
	printk(KERN_WARNING "STARTED ext3_open_file\n");
	mutex_lock(&ext3_open_pids->lock);
	ext3_open_pids->process_pids[ext3_open_pids->num_pids] = current->pid;
	ext3_open_pids->num_pids++;
	mutex_unlock(&ext3_open_pids->lock);
	
	err = dquot_file_open(inode, file);
	printk(KERN_WARNING "FINISHED ext3_open_file\n");
	return err;
}

/*
 * Called when an inode is released. Note that this is different
 * from ext3_file_open: open gets called at every open, but release
 * gets called only when /all/ the files are closed.
 */
static int ext3_release_file (struct inode * inode, struct file * filp)
{
	printk(KERN_WARNING "STARTED ext3_release_file\n");
	if (ext3_test_inode_state(inode, EXT3_STATE_FLUSH_ON_CLOSE)) {
		filemap_flush(inode->i_mapping);
		ext3_clear_inode_state(inode, EXT3_STATE_FLUSH_ON_CLOSE);
	}
	/* if we are the last writer on the inode, drop the block reservation */
	if ((filp->f_mode & FMODE_WRITE) &&
			(atomic_read(&inode->i_writecount) == 1))
	{
		mutex_lock(&EXT3_I(inode)->truncate_mutex);
		ext3_discard_reservation(inode);
		mutex_unlock(&EXT3_I(inode)->truncate_mutex);
	}
	if (is_dx(inode) && filp->private_data)
		ext3_htree_free_dir_info(filp->private_data);
	printk(KERN_WARNING "FINISHED ext3_release_file\n");
	return 0;
}

const struct file_operations ext3_file_operations = {
	.llseek		= generic_file_llseek,
	.read		= do_sync_read,
	.write		= do_sync_write,
	.aio_read	= ext3_read,
	.aio_write	= ext3_write,
	.unlocked_ioctl	= ext3_ioctl,
#ifdef CONFIG_COMPAT
	.compat_ioctl	= ext3_compat_ioctl,
#endif
	.mmap		= generic_file_mmap,
	.open		= ext3_open_file,
	.release	= ext3_release_file,
	.fsync		= ext3_sync_file,
	.splice_read	= generic_file_splice_read,
	.splice_write	= generic_file_splice_write,
};

const struct inode_operations ext3_file_inode_operations = {
	.setattr	= ext3_setattr,
#ifdef CONFIG_EXT3_FS_XATTR
	.setxattr	= generic_setxattr,
	.getxattr	= generic_getxattr,
	.listxattr	= ext3_listxattr,
	.removexattr	= generic_removexattr,
#endif
	.get_acl	= ext3_get_acl,
	.fiemap		= ext3_fiemap,
};

