#!/usr/bin/python


######################################################################
# Calls all of the calculators, decides on the security level,
# and passes level to the file system.
#
# 1. Load all security modules and get moduleOutput
#      a. Config file?
#           Keep all secMods in one directory
#      b. Call standard function to get moduleOutput
# 2. Load correct version(s) of calculator and calculate security level
# 3. Pass level to the file system
#      a. IOCTL
#      !WILL NOT WORK!  b. proc file
# NOTES: Leave modules and calculators outside context monitor
#        Allows both to be extended/changed
######################################################################

# Divide modules into sets by Calc version
# Pass each calculator its respective set of moduleOutputs
#
# Each secModule needs function: SecModuleOutput(), returns moduleOutput object.
# Use // var = __import__('modName')
# Read strings from config file, and import/call the SecModuleOutput() function
#   from each module in config file.

import sys
import os

from moduleOutputSetClass import moduleOutputSet
from moduleOutputClass import moduleOutput

# If an secModule cannot be found, halt the process!
#   Catch exception and proceed without it?
# Stop if there are no modules
# Security modules cannot be classes -> cannot import
def readConfigFile():
  f = open('CONFIG', 'r')
  moduleList = []

  i = 0
  for line in f:
    # If it starts with '#' ignore it.
    if line[0] != '#':
      if line[-1] == '\n':
        tempMod = line[:-1] # Remove '\n' character.
      else:
        tempMod = line

      moduleList.append(tempMod)

  f.close()
  return moduleList

# Preface each element in a List with the given prefix.
def prefaceList(prefix,List):
  for index, item in enumerate(List):
    temp = List[index]
    List[index] = prefix + temp
  return List

# Import every module in CONFIG file and call its SecModuleOutput function.
# Store results in moduleInfo data structure and return structure.
def getModuleInfo(moduleList):

  moduleInfoSet = moduleOutputSet()

  for item in moduleList:
    #print item
    var = __import__(item, globals(), locals(), ['SecModuleOutput'],-1)
    #print var
    tempArray = var.SecModuleOutput()
    #print tempArray

    modInfo = moduleOutput()
    for element in tempArray:
      modInfo.addSecInfo(element)

    moduleInfoSet.addModuleOutput(modInfo)
    #print modInfo.getSecInfo()


  #print 'test1'
  #temptest = moduleInfoSet.getModuleOutput(0)
  #print temptest.getSecInfo()

  return moduleInfoSet

# Input set of information gathered from modules.
# Output array of security levels calculated.
def getCalcInfo(moduleInfoSet):

  #print 'test3'
  #temptest = moduleInfoSet.getModuleOutput(0)
  #print temptest.getSecInfo()

  secLevelArray = []

  size = moduleInfoSet.getSize()
  for i in range(size):
    tempModInfo = moduleInfoSet.getModuleOutput(i)
    tempSecInfo = tempModInfo.getSecInfo()
    #print tempSecInfo[0]
    calcFunction = 'SecCalc.' + tempSecInfo[0]
    #print calcFunction
    secFunction = __import__(calcFunction, globals(), locals(), ['SecCalcOutput'],-1)
    secLevelArray.append(secFunction.SecCalcOutput(tempSecInfo))

  return secLevelArray

# Input set of output from security modules.
# Call each calculator and return set of security levels.
def calcSecLevel(modInfoSet):

  secLevelList = getCalcInfo(modInfoSet)
  return min(secLevelList)

#  secLevels = []
#  size = modInfo.getSize()
#  print size
#  for i in range(size):
#    print i


def main():

  # Read the CONFIG file containing security modules.
  moduleList = readConfigFile()
  #print moduleList
  numMod = len(moduleList)
  #print numMod

  # Add prefix 'SecModule.' to all module names.
  moduleList = prefaceList("SecModule.", moduleList)
  #print moduleList

  #Returns data structure of type moduleOutputSet
  moduleInfoSet = getModuleInfo(moduleList)

  #print 'test2'
  #temptest = moduleInfoSet.getModuleOutput(0)
  #print temptest.getSecInfo()

  contextLevel = calcSecLevel(moduleInfoSet)
  print contextLevel

  

if __name__=='__main__':
  main()
